@@ -5,27 +5,21 @@
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import os
-from os import path
 import warnings
+from os import path
 
-try:
-    import multiprocessing
-except ImportError:
-    multiprocessing = None
-
-from six import itervalues
 from docutils import nodes
 
 from sphinx.deprecation import RemovedInSphinx20Warning
-from sphinx.util import i18n, path_stabilize, logging, status_iterator
-from sphinx.util.osutil import SEP, relative_uri
-from sphinx.util.i18n import find_catalog
+from sphinx.environment.adapters.asset import ImageAdapter
+from sphinx.util import i18n, logging, status_iterator
 from sphinx.util.console import bold  # type: ignore
+from sphinx.util.i18n import find_catalog
+from sphinx.util.osutil import SEP, ensuredir, relative_uri
 from sphinx.util.parallel import ParallelTasks, SerialTasks, make_chunks, \
     parallel_available
 
@@ -33,6 +27,11 @@ from sphinx.util.parallel import ParallelTasks, SerialTasks, make_chunks, \
 from sphinx import roles       # noqa
 from sphinx import directives  # noqa
 
+try:
+    import multiprocessing
+except ImportError:
+    multiprocessing = None
+
 if False:
     # For type annotation
     from typing import Any, Callable, Dict, Iterable, List, Sequence, Set, Tuple, Union  # NOQA
@@ -55,6 +54,11 @@ class Builder(object):
     name = ''  # type: unicode
     #: The builder's output format, or '' if no document output is produced.
     format = ''  # type: unicode
+    #: The message emitted upon successful build completion. This can be a
+    #: printf-style template string with the following keys: ``outdir``,
+    #: ``project``
+    epilog = ''  # type: unicode
+
     # default translator class for the builder.  This will be overrided by
     # ``app.set_translator()``.
     default_translator_class = None  # type: nodes.NodeVisitor
@@ -78,8 +82,7 @@ class Builder(object):
         self.confdir = app.confdir
         self.outdir = app.outdir
         self.doctreedir = app.doctreedir
-        if not path.isdir(self.doctreedir):
-            os.makedirs(self.doctreedir)
+        ensuredir(self.doctreedir)
 
         self.app = app              # type: Sphinx
         self.env = None             # type: BuildEnvironment
@@ -91,9 +94,6 @@ class Builder(object):
         self.tags.add(self.name)
         self.tags.add("format_%s" % self.format)
         self.tags.add("builder_%s" % self.name)
-        # compatibility aliases
-        self.status_iterator = app.status_iterator
-        self.old_status_iterator = app.old_status_iterator
 
         # images that need to be copied over (source -> dest)
         self.images = {}  # type: Dict[unicode, unicode]
@@ -200,6 +200,7 @@ class Builder(object):
     def post_process_images(self, doctree):
         # type: (nodes.Node) -> None
         """Pick the best candidate for all image URIs."""
+        images = ImageAdapter(self.env)
         for node in doctree.traverse(nodes.image):
             if '?' in node['candidates']:
                 # don't rewrite nonlocal image URIs
@@ -210,8 +211,14 @@ class Builder(object):
                     if candidate:
                         break
                 else:
-                    logger.warning('no matching candidate for image URI %r', node['uri'],
-                                   location=node)
+                    mimetypes = sorted(node['candidates'])
+                    image_uri = images.get_original_image_uri(node['uri'])
+                    if mimetypes:
+                        logger.warning('a suitable image for %s builder not found: %s (%s)',
+                                       self.name, mimetypes, image_uri, location=node)
+                    else:
+                        logger.warning('a suitable image for %s builder not found: %s',
+                                       self.name, image_uri, location=node)
                     continue
                 node['uri'] = candidate
             else:
@@ -253,11 +260,14 @@ class Builder(object):
         # type: (List[unicode]) -> None
         def to_domain(fpath):
             # type: (unicode) -> unicode
-            docname, _ = path.splitext(path_stabilize(fpath))
-            dom = find_catalog(docname, self.config.gettext_compact)
-            return dom
+            docname = self.env.path2doc(path.abspath(fpath))
+            if docname:
+                return find_catalog(docname, self.config.gettext_compact)
+            else:
+                return None
 
         specified_domains = set(map(to_domain, specified_files))
+        specified_domains.discard(None)
         catalogs = i18n.find_catalog_source_files(
             [path.join(self.srcdir, x) for x in self.config.locale_dirs],
             self.config.language,
@@ -373,15 +383,10 @@ class Builder(object):
             docnames = set(docnames) & self.env.found_docs
 
         # determine if we can write in parallel
-        self.parallel_ok = False
         if parallel_available and self.app.parallel > 1 and self.allow_parallel:
-            self.parallel_ok = True
-            for extension in itervalues(self.app.extensions):
-                if not extension.parallel_write_safe:
-                    logger.warning('the %s extension is not safe for parallel '
-                                   'writing, doing serial write', extension.name)
-                    self.parallel_ok = False
-                    break
+            self.parallel_ok = self.app.is_parallel_allowed('write')
+        else:
+            self.parallel_ok = False
 
         #  create a task executor to use for misc. "finish-up" tasks
         # if self.parallel_ok:
