@@ -17,7 +17,6 @@ import re
 import sys
 import time
 from os import path
-from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
@@ -33,7 +32,8 @@ from sphinx.util.console import bold  # type: ignore
 from sphinx.util.nodes import set_source_info
 from sphinx.util.osutil import fs_encoding
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Callable, Dict, IO, Iterable, List, Optional, Sequence, Set, Tuple  # NOQA
     from sphinx.application import Sphinx  # NOQA
 
